#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "count_list.h"
/* A module to store words and their associated counts in an array.  The array will contain 
   structs.  Each struct represents on word.  */

/* The maximum number of entries in a word list. */
#define MAX_WORDS 10000

/* Information about one word. */
struct entry {
  char *word;
  int count;
};

/* The list */
static struct entry *word_list[MAX_WORDS];

/* The number of entries currently in the list. */
static int n_words;

/* Initially, the list has no entries in it. */
void initialize_word_list()
{
  n_words = 0;
}


/* Search for a word in the list.  Return true if the
   word is in the list, false otherwise. */
int in_list (char *word)
{
  int i;
  for (i = 0; i < n_words; i++) {
    if (strcmp (word, word_list[i]->word) == 0) {
      return 1;
    }
  }
  /* Didn't find it. */
  return 0;
}


/* Add a new word to the list, and initialize its count to 0.  Return
   1 on success, 0 on failure. */
int add_word (char *word)
{
  if (n_words == MAX_WORDS) {
    /* Failed to add the word to the list. */
    return 0;
  }

  /* Create a new entry in the list. 
     First, allocate memory for the entry. */
  struct entry *new_entry =
    (struct entry *) malloc (sizeof (struct entry));
  
  /* Now, allocate memory for the word, and copy the word into this
     new memory. */
  new_entry->word = (char *) malloc (strlen (word) + 1);
  strcpy (new_entry->word, word);
  
  /* Set the word count to 1. */
  new_entry->count = 1;
  
  /* Add the word to the list. */
  word_list[n_words] = new_entry;
  
  /* Up word count */
  n_words++;
  
  /* We succeeded in adding the word to the list. */
  return 1;
}


/* If the word pointed to by word is in the list, increment its count
   and return the new count.  If it is not in the list, return 0. */
int increment_word_count (char *word)
{
  int i;
  
  for (i = 0; i < n_words; i++) {
    if (strcmp (word, word_list[i]->word) == 0) {
      return ++word_list[i]->count;
    }
  }
  return 0;
}


/* Print all the words and their frequences. */
void print_list ()
{
  int i;
  
  printf ("Word\tFrequency\n");
  printf ("----\t---------\n");
  for (i = 0; i < n_words; i++) {
    printf ("%s\t%9d\n", word_list[i]->word, word_list[i]->count);
  }

}


/* Release all the memory occupied by the list. */
void cleanup_list()
{
  int i;

  for (i = 0; i < n_words; i++) {
    free (word_list[i]->word);
    free (word_list[i]);
  }
}



